@@ -34,7 +34,7 @@ class HiveApiAsync:
             "devices": f"{self.baseUrl}/devices",
             "products": f"{self.baseUrl}/products",
             "actions": f"{self.baseUrl}/actions",
-            "nodes": f"{self.baseUrl}/nodes/{0}/{1}",
+            "nodes": f"{self.baseUrl}/nodes/{{0}}/{{1}}",
             "long_lived": "https://api.prod.bgchprod.info/omnia/accessTokens",
             "weather": "https://weather.prod.bgchprod.info/weather",
         }
@@ -112,7 +112,7 @@ class HiveApiAsync:
         return loginData
 
     async def refreshTokens(self):
-        """Refresh tokens."""
+        """Refresh tokens - DEPRECATED NOW BY AWS TOKEN MANAGEMENT."""
         url = self.urls["refresh"]
         if self.session is not None:
             tokens = self.session.tokens.tokenData
