@@ -7,6 +7,7 @@ import hashlib
 import hmac
 import os
 import re
+import socket
 
 import boto3
 import botocore
@@ -14,6 +15,7 @@ import botocore
 from ..helper.hive_exceptions import (
     HiveApiError,
     HiveInvalid2FACode,
+    HiveInvalidDeviceAuthentication,
     HiveInvalidPassword,
     HiveInvalidUsername,
 )
@@ -66,11 +68,15 @@ class HiveAuth:
     NEW_PASSWORD_REQUIRED_CHALLENGE = "NEW_PASSWORD_REQUIRED"
     PASSWORD_VERIFIER_CHALLENGE = "PASSWORD_VERIFIER"
     SMS_MFA_CHALLENGE = "SMS_MFA"
+    DEVICE_VERIFIER_CHALLENGE = "DEVICE_SRP_AUTH"
 
     def __init__(
         self,
         username: str,
         password: str,
+        deviceGroupKey: str = None,
+        deviceKey: str = None,
+        devicePassword: str = None,
         pool_region: str = None,
         client_secret: str = None,
     ):
@@ -93,6 +99,10 @@ class HiveAuth:
 
         self.username = username
         self.password = password
+        self.deviceGroupKey = deviceGroupKey
+        self.deviceKey = deviceKey
+        self.devicePassword = devicePassword
+        self.accessToken = None
         self.user_id = "user_id"
         self.client_secret = client_secret
         self.big_n = hex_to_long(n_hex)
@@ -189,6 +199,96 @@ class HiveAuth:
         hmac_obj = hmac.new(bytearray(client_secret, "utf-8"), message, hashlib.sha256)
         return base64.standard_b64encode(hmac_obj.digest()).decode("utf-8")
 
+    def generate_hash_device(self, device_group_key, device_key):
+        """Generate the device hash."""
+        # source: https://github.com/amazon-archives/amazon-cognito-identity-js/blob/6b87f1a30a998072b4d98facb49dcaf8780d15b0/src/AuthenticationHelper.js#L137
+
+        # random device password, which will be used for DEVICE_SRP_AUTH flow
+        device_password = base64.standard_b64encode(os.urandom(40)).decode("utf-8")
+
+        combined_string = f"{device_group_key}{device_key}:{device_password}"
+        combined_string_hash = hash_sha256(combined_string.encode("utf-8"))
+        salt = pad_hex(get_random(16))
+
+        x_value = hex_to_long(hex_hash(salt + combined_string_hash))
+        g = hex_to_long(g_hex)
+        big_n = hex_to_long(n_hex)
+        verifier_device_not_padded = pow(g, x_value, big_n)
+        verifier = pad_hex(verifier_device_not_padded)
+
+        device_secret_verifier_config = {
+            "PasswordVerifier": base64.standard_b64encode(
+                bytearray.fromhex(verifier)
+            ).decode("utf-8"),
+            "Salt": base64.standard_b64encode(bytearray.fromhex(salt)).decode("utf-8"),
+        }
+        return device_password, device_secret_verifier_config
+
+    def get_device_authentication_key(
+        self, device_group_key, device_key, device_password, server_b_value, salt
+    ):
+        """Get the device authentication key."""
+        u_value = calculate_u(self.large_a_value, server_b_value)
+        if u_value == 0:
+            raise ValueError("U cannot be zero.")
+        username_password = f"{device_group_key}{device_key}:{device_password}"
+        username_password_hash = hash_sha256(username_password.encode("utf-8"))
+
+        x_value = hex_to_long(hex_hash(pad_hex(salt) + username_password_hash))
+        g_mod_pow_xn = pow(self.g, x_value, self.big_n)
+        int_value2 = server_b_value - self.k * g_mod_pow_xn
+        s_value = pow(int_value2, self.small_a_value + u_value * x_value, self.big_n)
+        hkdf = compute_hkdf(
+            bytearray.fromhex(pad_hex(s_value)),
+            bytearray.fromhex(pad_hex(long_to_hex(u_value))),
+        )
+        return hkdf
+
+    def process_device_challenge(self, challenge_parameters):
+        """Process the device challenge."""
+        username = challenge_parameters["USERNAME"]
+        salt_hex = challenge_parameters["SALT"]
+        srp_b_hex = challenge_parameters["SRP_B"]
+        secret_block_b64 = challenge_parameters["SECRET_BLOCK"]
+        # re strips leading zero from a day number (required by AWS Cognito)
+        timestamp = re.sub(
+            r" 0(\d) ",
+            r" \1 ",
+            datetime.datetime.utcnow().strftime("%a %b %d %H:%M:%S UTC %Y"),
+        )
+        hkdf = self.get_device_authentication_key(
+            self.deviceGroupKey,
+            self.deviceKey,
+            self.devicePassword,
+            hex_to_long(srp_b_hex),
+            salt_hex,
+        )
+        secret_block_bytes = base64.standard_b64decode(secret_block_b64)
+        msg = (
+            bytearray(self.deviceGroupKey, "utf-8")
+            + bytearray(self.deviceKey, "utf-8")
+            + bytearray(secret_block_bytes)
+            + bytearray(timestamp, "utf-8")
+        )
+        hmac_obj = hmac.new(hkdf, msg, digestmod=hashlib.sha256)
+        signature_string = base64.standard_b64encode(hmac_obj.digest())
+        response = {
+            "TIMESTAMP": timestamp,
+            "USERNAME": username,
+            "PASSWORD_CLAIM_SECRET_BLOCK": secret_block_b64,
+            "PASSWORD_CLAIM_SIGNATURE": signature_string.decode("utf-8"),
+            "DEVICE_KEY": self.deviceKey,
+        }
+        if self.client_secret is not None:
+            response.update(
+                {
+                    "SECRET_HASH": self.get_secret_hash(
+                        username, self.__client_id, self.client_secret
+                    )
+                }
+            )
+        return response
+
     def process_challenge(self, challenge_parameters: dict):
         """Process 2FA challenge."""
         self.user_id = challenge_parameters["USER_ID_FOR_SRP"]
@@ -236,6 +336,8 @@ class HiveAuth:
             return self.file_response
 
         auth_params = self.get_auth_params()
+        if self.deviceKey is not None:
+            auth_params["DEVICE_KEY"] = self.deviceKey
         response = None
         result = None
         try:
@@ -253,6 +355,9 @@ class HiveAuth:
 
         if response["ChallengeName"] == self.PASSWORD_VERIFIER_CHALLENGE:
             challenge_response = self.process_challenge(response["ChallengeParameters"])
+            if self.deviceKey is not None:
+                challenge_response["DEVICE_KEY"] = self.deviceKey
+
             try:
                 result = self.client.respond_to_auth_challenge(
                     ClientId=self.__client_id,
@@ -262,6 +367,8 @@ class HiveAuth:
             except botocore.exceptions.ClientError as err:
                 if err.__class__.__name__ == "NotAuthorizedException":
                     raise HiveInvalidPassword
+                if err.__class__.__name__ == "ResourceNotFoundException":
+                    raise HiveInvalidDeviceAuthentication
             except botocore.exceptions.EndpointConnectionError as err:
                 if err.__class__.__name__ == "EndpointConnectionError":
                     raise HiveApiError
@@ -272,7 +379,41 @@ class HiveAuth:
                 "The %s challenge is not supported" % response["ChallengeName"]
             )
 
-    def sms_2fa(self, entered_code: str, challenge_parameters: dict):
+    def deviceLogin(self):
+        """Perform device login instead."""
+        loginResult = self.login()
+        auth_params = self.get_auth_params()
+        auth_params["DEVICE_KEY"] = self.deviceKey
+
+        if loginResult.get("ChallengeName") == self.DEVICE_VERIFIER_CHALLENGE:
+            try:
+                initialResult = self.client.respond_to_auth_challenge(
+                    ClientId=self.__client_id,
+                    ChallengeName=self.DEVICE_VERIFIER_CHALLENGE,
+                    ChallengeResponses=auth_params,
+                )
+
+                cr = self.process_device_challenge(initialResult["ChallengeParameters"])
+                result = self.client.respond_to_auth_challenge(
+                    ClientId=self.__client_id,
+                    ChallengeName="DEVICE_PASSWORD_VERIFIER",
+                    ChallengeResponses=cr,
+                )
+            except botocore.exceptions.EndpointConnectionError as err:
+                if err.__class__.__name__ == "EndpointConnectionError":
+                    raise HiveApiError
+        else:
+            raise HiveInvalidDeviceAuthentication
+
+        return result
+
+    def sms_2fa(
+        self,
+        entered_code: str,
+        challenge_parameters: dict,
+        deviceName=None,
+        autoDeviceRegistration=True,
+    ):
         """Process 2FA sms verification."""
         session = challenge_parameters.get("Session")
         code = str(entered_code)
@@ -287,6 +428,19 @@ class HiveAuth:
                     "USERNAME": self.user_id,
                 },
             )
+            if "NewDeviceMetadata" in result["AuthenticationResult"]:
+                self.accessToken = result["AuthenticationResult"]["AccessToken"]
+                self.deviceGroupKey = result["AuthenticationResult"][
+                    "NewDeviceMetadata"
+                ]["DeviceGroupKey"]
+                self.deviceKey = result["AuthenticationResult"]["NewDeviceMetadata"][
+                    "DeviceKey"
+                ]
+            if autoDeviceRegistration:
+                self.confirmDevice(
+                    self.accessToken, self.deviceKey, self.deviceGroupKey, deviceName
+                )
+                self.updateDeviceStatus(self.accessToken)
         except botocore.exceptions.ClientError as err:
             if (
                 err.__class__.__name__ == "NotAuthorizedException"
@@ -299,19 +453,74 @@ class HiveAuth:
 
         return result
 
+    def confirmDevice(
+        self,
+        accessToken: str,
+        deviceKey: str,
+        deviceGroupKey: str,
+        deviceName: str = None,
+    ):
+        """Confirm Device Hive."""
+        result = None
+        if deviceName is None:
+            deviceName = socket.gethostname()
+        try:
+            device_password, device_secret_verifier_config = self.generate_hash_device(
+                deviceGroupKey, deviceKey
+            )
+            self.devicePassword = device_password
+            result = (
+                self.client.confirm_device(
+                    AccessToken=accessToken,
+                    DeviceKey=deviceKey,
+                    DeviceName=deviceName,
+                    DeviceSecretVerifierConfig=device_secret_verifier_config,
+                ),
+            )
+        except botocore.exceptions.EndpointConnectionError as err:
+            if err.__class__.__name__ == "EndpointConnectionError":
+                raise HiveApiError
+
+        return result
+
+    def updateDeviceStatus(
+        self,
+        accessToken: str,
+    ):
+        """Update Device Hive."""
+        result = None
+        try:
+            result = (
+                self.client.update_device_status(
+                    AccessToken=accessToken,
+                    DeviceKey=self.deviceKey,
+                    DeviceRememberedStatus="remembered",
+                ),
+            )
+        except botocore.exceptions.EndpointConnectionError as err:
+            if err.__class__.__name__ == "EndpointConnectionError":
+                raise HiveApiError
+
+        return result
+
+    def getDeviceData(self):
+        """Get key device information to use device authentication."""
+        return self.deviceGroupKey, self.deviceKey, self.devicePassword
+
     def refreshToken(
         self,
         refresh_token: str,
     ):
         """Refresh Hive Tokens."""
         result = None
+        auth_params = ({"REFRESH_TOKEN": refresh_token},)
+        if self.deviceKey is not None:
+            auth_params = {"REFRESH_TOKEN": refresh_token, "DEVICE_KEY": self.deviceKey}
         try:
-            result = (
-                self.client.initiate_auth(
-                    ClientId=self.__client_id,
-                    AuthFlow="REFRESH_TOKEN_AUTH",
-                    AuthParameters={"REFRESH_TOKEN": refresh_token},
-                ),
+            result = self.client.initiate_auth(
+                ClientId=self.__client_id,
+                AuthFlow="REFRESH_TOKEN_AUTH",
+                AuthParameters=auth_params,
             )
         except botocore.exceptions.EndpointConnectionError as err:
             if err.__class__.__name__ == "EndpointConnectionError":
