@@ -15,6 +15,11 @@ from .device_attributes import HiveAttributes
 from .helper.const import ACTIONS, DEVICES, HIVE_TYPES, PRODUCTS
 from .helper.hive_exceptions import (
     HiveApiError,
+    HiveFailedToRefreshTokens,
+    HiveInvalid2FACode,
+    HiveInvalidDeviceAuthentication,
+    HiveInvalidPassword,
+    HiveInvalidUsername,
     HiveReauthRequired,
     HiveUnknownConfiguration,
     NoApiToken,
@@ -40,7 +45,13 @@ class HiveSession:
     sessionType = "Session"
 
     def __init__(
-        self, username: str = None, password: str = None, websession: object = None
+        self,
+        username: str = None,
+        password: str = None,
+        deviceGroupKey: str = None,
+        deviceKey: str = None,
+        devicePassword: str = None,
+        websession: object = None,
     ):
         """Initialise the base variable values.
 
@@ -49,7 +60,13 @@ class HiveSession:
             password (str, optional): Hive Password. Defaults to None.
             websession (object, optional): Websession for api calls. Defaults to None.
         """
-        self.auth = Auth(username=username, password=password)
+        self.auth = Auth(
+            username=username,
+            password=password,
+            deviceGroupKey=deviceGroupKey,
+            deviceKey=deviceKey,
+            devicePassword=devicePassword,
+        )
         self.api = API(hiveSession=self, websession=websession)
         self.helper = HiveHelper(self)
         self.attr = HiveAttributes(self)
@@ -192,6 +209,7 @@ class HiveSession:
             if "RefreshToken" in data:
                 self.tokens.tokenData.update({"refreshToken": data["RefreshToken"]})
             self.tokens.tokenData.update({"accessToken": data["AccessToken"]})
+            self.tokens.tokenCreated = datetime.now()
         elif "token" in tokens:
             data = tokens
             self.tokens.tokenData.update({"token": data["token"]})
@@ -210,27 +228,71 @@ class HiveSession:
             HiveUnknownConfiguration: Login information is unknown.
 
         Returns:
-            dict: result of the login request.
+            dict: result of the authentication request.
         """
+        result = None
         if not self.auth:
             raise HiveUnknownConfiguration
 
-        result = self.auth.login()
-        await self.updateTokens(result)
+        try:
+            result = await self.auth.login()
+        except HiveInvalidUsername:
+            print("invalid_username")
+        except HiveInvalidPassword:
+            print("invalid_password")
+        except HiveApiError:
+            print("no_internet_available")
+
+        if "AuthenticationResult" in result:
+            await self.updateTokens(result)
         return result
 
-    async def sms2FA(self, code: str, session: dict):
-        """Complete 2FA auth.
+    async def sms2fa(self, code, session, deviceName=None):
+        """Login to hive account with 2 factor authentication.
 
-        Args:
-            code (str): 2FA code to complete login.
-            session (dict): The session data from login.
+        Raises:
+            HiveUnknownConfiguration: Login information is unknown.
 
         Returns:
-            dict: result of the login request.
+            dict: result of the authentication request.
         """
-        result = self.auth.sms_2fa(code, session)
-        await self.updateTokens(result)
+        result = None
+        if not self.auth:
+            raise HiveUnknownConfiguration
+
+        try:
+            result = await self.auth.sms_2fa(code, session, deviceName)
+        except HiveInvalid2FACode:
+            print("invalid_code")
+        except HiveApiError:
+            print("no_internet_available")
+
+        if "AuthenticationResult" in result:
+            await self.updateTokens(result)
+        return result
+
+    async def deviceLogin(self):
+        """Login to hive account using device authentication.
+
+        Raises:
+            HiveUnknownConfiguration: Login information is unknown.
+            HiveInvalidDeviceAuthentication: Device information is unknown.
+
+        Returns:
+            dict: result of the authentication request.
+        """
+        result = None
+        if not self.auth:
+            raise HiveUnknownConfiguration
+
+        try:
+            result = await self.auth.deviceLogin()
+        except HiveInvalidDeviceAuthentication:
+            raise HiveInvalidDeviceAuthentication
+
+        if "AuthenticationResult" in result:
+            await self.updateTokens(result)
+            self.tokens.tokenExpiry = timedelta(seconds=0)
         return result
 
     async def hiveRefreshTokens(self):
@@ -249,8 +311,11 @@ class HiveSession:
                 result = await self.auth.refreshToken(
                     self.tokens.tokenData["refreshToken"]
                 )
-                await self.updateTokens(result)
-                self.tokens.tokenCreated = datetime.now()
+
+                if "AuthenticationResult" in result:
+                    await self.updateTokens(result)
+                else:
+                    raise HiveFailedToRefreshTokens
 
         return result
 
@@ -303,27 +368,38 @@ class HiveSession:
             HTTPException: HTTP error has occurred updating the devices.
             HiveApiError: An API error code has been returned.
         """
+        cameraImage = None
+        cameraRecording = None
         if self.config.file:
             cameraImage = self.openFile("camera.json")
-        if self.tokens is None:
+            cameraRecording = self.openFile("camera.json")
+        elif self.tokens is not None:
+            cameraImage = await self.api.getCameraImage(device)
+            if cameraImage["parsed"]["events"][0]["hasRecording"] is True:
+                cameraRecording = await self.api.getCameraRecording(
+                    device, cameraImage["parsed"]["events"][0]["eventId"]
+                )
+
+            if operator.contains(str(cameraImage["original"]), "20") is False:
+                raise HTTPException
+            elif cameraImage["parsed"] is None:
+                raise HiveApiError
+        else:
             raise NoApiToken
-        cameraImage = await self.api.getCameraImage(device)
-        if cameraImage["parsed"]["events"][0]["hasRecording"] is True:
-            cameraRecording = await self.api.getCameraRecording(
-                device, cameraImage["parsed"]["events"][0]["eventId"]
-            )
 
-        if operator.contains(str(cameraImage["original"]), "20") is False:
-            raise HTTPException
-        elif cameraImage["parsed"] is None:
-            raise HiveApiError
         self.data.camera[device["id"]] = {}
-        self.data.camera[device["id"]]["cameraImage"] = cameraImage["parsed"]["events"][
-            0
-        ]
-        self.data.camera[device["id"]]["cameraRecording"] = cameraRecording["parsed"]
+        self.data.camera[device["id"]]["cameraImage"] = None
+        self.data.camera[device["id"]]["cameraRecording"] = None
 
-        return self.data.camera[device["id"]]
+        if cameraImage is not None:
+            self.data.camera[device["id"]] = {}
+            self.data.camera[device["id"]]["cameraImage"] = cameraImage["parsed"][
+                "events"
+            ][0]
+        if cameraRecording is not None:
+            self.data.camera[device["id"]]["cameraRecording"] = cameraRecording[
+                "parsed"
+            ]
 
     async def getDevices(self, n_id: str):
         """Get latest data for Hive nodes.
